@@ -37,7 +37,8 @@ enum {
   STATUS_NOT_DRAG,
   STATUS_IS_DRAG,
   STATUS_CLICK,
-  STATUS_SHOW
+  STATUS_SHOW,
+  STATUS_MARKED,
 };
 
 #endif
