@@ -125,8 +125,42 @@
        (not (empty-slot? 0))
        (deal-cards-face-up 0 '(1))))
 
+(define (move-to-foundation source target)
+  (cond ((eq? target '()) #f)
+	((and (not (empty-slot? (car target)))
+	      (eq? (get-suit (get-top-card source))
+		   (get-suit (get-top-card (car target))))
+	      (or (= (+ 1 (get-value (get-top-card (car target))))
+		     (get-value (get-top-card source)))
+		  (and (= (get-value (get-top-card (car target))) king)
+		       (= (get-value (get-top-card source)) ace))))
+	 (add-card! (car target) (remove-card source))
+	 (add-to-score! 1)
+	 #t)
+	(#t (move-to-foundation source (cdr target)))))
+  
 (define (button-double-clicked slot-id)
-  #f)
+  (cond ((or (empty-slot? slot-id)
+             (= slot-id 0)
+             (= slot-id 2)
+             (= slot-id 4)
+             (= slot-id 8)
+             (> slot-id 9))
+         #f)
+        ((= (get-value (get-top-card slot-id)) BASE-VAL)
+         (let ((top-card (get-top-card slot-id)))
+           (remove-card slot-id)
+           (cond ((empty-slot? 4)
+                  (move-n-cards! slot-id 4 (list top-card)))
+                 ((empty-slot? 8)
+                  (move-n-cards! slot-id 8 (list top-card)))
+                 (#t
+                  (move-n-cards! slot-id 10 (list top-card))))
+           (add-to-score! 1))
+	 #t)
+	((move-to-foundation slot-id '(2 4 8 10)) 
+	 #t)
+        (#t #f)))
 
 (define (game-continuable)
   (give-status-message)
