@@ -51,8 +51,6 @@ GdkPixmap *tiles_mask = NULL;
 
 gint flip_pixmaps_id = 0;
 gint statusbar_id;
-guint whose_turn = BLACK_TURN;
-guint game_in_progress = 0;
 guint black_computer_level;
 guint white_computer_level;
 guint black_computer_id = 0;
@@ -231,6 +229,7 @@ static void undo_set_sensitive (gboolean state)
 void
 quit_game_cb (GtkWidget *widget, gpointer data)
 {
+  games_kill_server ();
   gtk_main_quit ();
 }
 
@@ -535,7 +534,7 @@ load_pixmaps (void)
 
 	if (! g_file_test (fname,
 			   G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) {
-		g_print (_("Could not find \'%s\' pixmap file for Iagno\n"), fname);
+		g_print (_("Could not find \'%s\' pixmap file\n"), fname);
 		exit (1);
 	}
 
@@ -680,6 +679,7 @@ init_new_game (void)
 	clear_board ();
 	game_in_progress = 1;
 	move_count = 0;
+	whose_turn = BLACK_TURN;
 
 	undo_set_sensitive (FALSE);
 	
@@ -800,7 +800,7 @@ gui_status (void)
 	gtk_label_set_text (GTK_LABEL (black_score), message);
 	sprintf (message, _("%.2d"), wcount);
 	gtk_label_set_text (GTK_LABEL (white_score), message);
-	undo_set_sensitive (move_count > 0);
+	undo_set_sensitive (move_count > 0 && !is_network_running ());
 }
 
 void
